/* David Leonard, 2001. Public domain */
/* $Id: buffer.c,v 1.2 2002/02/07 01:09:24 d Exp $ */

/*
 * Circular i/o buffers.
 */

#include <unistd.h>
#include <stdlib.h>
#include <err.h>

#include "io.h"
#include "buffer.h"

int
buffer_read(io, b)
	struct io *io;
	struct buffer *b;
{
	int sz = IO_READ(io, b->data + buffer_readpos(b), 
		buffer_readsize(b));
	if (sz != -1)
		b->l += sz;
	return sz;
}

int
buffer_write(io, b)
	struct io *io;
	struct buffer *b;
{
	int sz = IO_WRITE(io, b->data + b->p, buffer_writesize(b));
	if (sz != -1) {
		b->l -= sz;
		if (b->l == 0)
			b->p = 0;
		else if (b->p + sz > b->m)
			b->p = b->p + sz - b->m;
		else
			b->p = b->p + sz;
	}
	return sz;
}

struct buffer *
buffer_new()
{
	struct buffer *b;

	b = (struct buffer *)malloc(sizeof *b);
	if (b == NULL) {
		warnx("malloc");
		return NULL;
	}
	b->m = 8192;
	b->data = malloc(b->m);
	if (b->data == NULL) {
		warnx("malloc");
		free(b);
		return NULL;
	}
	b->p = 0;
	b->l = 0;
	return b;
}

void
buffer_free(b)
	struct buffer *b;
{
	free(b->data);
	b->m = 0;
	b->data = NULL;
	free(b);
}

void
buffer_flush(b)
	struct buffer *b;
{
	b->l = 0;
	b->p = 0;
}
