/*
 * A circular buffer of fixed size.
 * Reads and writes do cross the boundary.
 */
/* $Id: buffer.h,v 1.2 2002/02/07 01:09:25 d Exp $ */

struct buffer {
	char	*data;
	int	p;		/* offset to head */
	int	l;		/* length of data in buffer (may wrap) */
	int	m;		/* maximum offset of tail */
};

/* Create a new buffer of indeterminate size */
struct	buffer *buffer_new();
/* Read from an io into tail of the buffer */
int	buffer_read(struct io*, struct buffer *);
/* Write from the head of the buffer to an io */
int	buffer_write(struct io*, struct buffer *);
/* Empty and reset the buffer */
void	buffer_flush(struct buffer *);
/* Destroy the buffer */
void	buffer_free(struct buffer *);

#undef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))

/* Return true if there is space in the buffer to do a buffer_read */
#define buffer_canread(b) \
	((b)->l < (b)->m)

/* Return true if there is content in the buffer to buffer_write */
#define buffer_canwrite(b) \
	((b)->l != 0)

/* Return the total amount of data in the buffer (may be multi-segment) */
#define buffer_len(b) \
	((b)->l)

/* Return the largest segment size that can be read atomically */
#define buffer_readsize(b) \
	(((b)->m - (b)->l) - (b)->p > 0 \
		? ((b)->m - (b)->l) - (b)->p \
		: (b)->m - (b)->l)
	
/* Return the largest segment size that can be written atomically */
#define buffer_writesize(b) \
	MIN((b)->l, (b)->m - (b)->p)

/* Return the offset into which to read in a new segment */
#define buffer_readpos(b) \
	(((b)->p + (b)->l) % (b)->m)

