/* David Leonard, 2001. Public domain. */
/* $Id: conn.c,v 1.2 2002/02/07 01:09:25 d Exp $ */

/*
 * A linked list of connections.
 */

#include <stdlib.h>
#include <err.h>
#include <sys/queue.h>
#include "io.h"
#include "conn.h"

struct connhead connhead = SLIST_HEAD_INITIALIZER(connhead);
int conns_changed = 0;

/*
 * Create a new TCP connection record with a given io.
 * It is automatically added to the 'connhead' list.
 */
struct conn *
conn_new(io)
	struct io *io;
{
	struct conn *conn = (struct conn *)malloc(sizeof *conn);

	if (conn == NULL) {
		warnx("malloc");
		return NULL;
	}
	SLIST_INSERT_HEAD(&connhead, conn, link);
	conns_changed++;
	conn->io = io;
	conn->closing = 0;
	return conn;
}

/*
 * Delete a TCP connection record and remove it from the 'connhead' list.
 * Caller must deallocate conn->io in advance.
 */
void
conn_free(conn)
	struct conn *conn;
{
	SLIST_REMOVE(&connhead, conn, conn, link);
	conns_changed++;
	free(conn);
}
