/* David Leonard, 2001. Public domain. */
/* $Id: iofd.c,v 1.2 2002/02/07 01:09:25 d Exp $ */

/*
 * Unix file descriptor i/o.
 */

#include <stdlib.h>
#include <unistd.h>
#include <poll.h>

#include "io.h"
#include "iofd.h"

struct iofd {
	struct io	io;
	int		fd;
};

/* NOTE: does NOT close the file descriptor */
static void
iofd_free(io)
	struct io *io;
{
	struct iofd *iofd = (struct iofd *)io;
	free(iofd);
};

static int
iofd_read(io, buf, sz)
	struct io *io;
	char *buf;
	int sz;
{
	struct iofd *iofd = (struct iofd *)io;
	return read(iofd->fd, buf, sz);
};

static int
iofd_write(io, buf, sz)
	struct io *io;
	char *buf;
	int sz;
{
	struct iofd *iofd = (struct iofd *)io;
	return write(iofd->fd, buf, sz);
};

static int
iofd_fileno(io)
	struct io *io;
{
	struct iofd *iofd = (struct iofd *)io;
	return iofd->fd;
};

static int
iofd_poll(io, pfd)
	struct io *io;
	struct pollfd *pfd;
{
	struct iofd *iofd = (struct iofd *)io;
	pfd->fd = iofd->fd;
	pfd->events |= POLLIN | POLLOUT;
	/* pfd->revents |= 0; */
	return 0;
};

static struct io_op iofd_op = { 
	"iofd",
	iofd_read, iofd_write, iofd_free, iofd_fileno, iofd_poll,
	0 /* not secure */
};

/* Create a new io for an open file descriptor */
struct io *
io_newfd(fd)
	int fd;
{
	struct iofd *iofd = (struct iofd *)malloc(sizeof *iofd);
	iofd->io.io_op = &iofd_op;
	iofd->fd = fd;
	return &iofd->io;
}
